/***
 * Fractal ADL Parser
 * Copyright (C) 2002-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.adl.bindings;

import java.io.PrintWriter;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * A Java based, static implementation of the {@link BindingBuilder} interface. 
 * This implementation produces standard Java code that binds components. 
 */

public class StaticJavaBindingBuilder implements BindingBuilder {

  /**
   * TODO.
   */
  
  private Map bindingLists = new WeakHashMap();
  
  // --------------------------------------------------------------------------
  // Implementation of the BindingBuilder interface
  // --------------------------------------------------------------------------
  
  public void bindComponent (
    final int type,
    final Object clientComp, 
    final String clientItf, 
    final Object serverComp, 
    final String serverItf, 
    final Object context)
  {
    Binding b = new Binding();
    b.client = (String)clientComp;
    b.clientInterface = clientItf;
    b.server = (String)serverComp;
    b.serverInterface = serverItf;
        
    List bindings = (List)bindingLists.get(context);
    if (bindings == null) {
      bindings = new ArrayList();
      bindingLists.put(context, bindings);
    }
    bindings.add(b);

    PrintWriter pw = (PrintWriter)((Map)context).get("printwriter");
    
    // generate code to create direct bindings between the primitive components
    for (int i = 0; i < bindings.size(); ++i) {
      b = (Binding)bindings.get(i);
      String client = (String)b.client;
      String clientInterface = b.clientInterface;
      String server = (String)b.server;
      String serverInterface = b.serverInterface;
      top: while (server != null && !server.startsWith("P")) {
        for (int j = 0; j < bindings.size(); ++j) {
          b = (Binding)bindings.get(j);
          if (b.client.equals(server) && b.clientInterface.equals(serverInterface)) {
            server = b.server;
            serverInterface = b.serverInterface;
            continue top;
          }
        }
        server = null;
      }
      if (server != null) {
        if (client.startsWith("P") ){
          pw.print(client);
          pw.print(".bindFc(\"");
          pw.print(clientInterface);
          pw.print("\",");
          pw.print(server);
          pw.println(");");
          bindings.remove(i--);
        } else {
          pw.print(client);
          pw.print(".put(\"");
          pw.print(clientInterface);
          pw.print("\",");
          pw.print(server);
          pw.println(");");
        }
      }
    }
  }  

  // --------------------------------------------------------------------------
  // Inner classes
  // --------------------------------------------------------------------------
  
  static class Binding {
    String client;
    String clientInterface;
    String server;
    String serverInterface;
  }
}
